@@ -174,6 +174,15 @@ gboolean cb_sc_follow(GtkEntry* entry, void* session);
  */
 gboolean cb_sc_display_link(GtkEntry* entry, void* session);
 
+/**
+ * Called when input has been passed to the sc_copy_link dialog
+ *
+ * @param entry The dialog inputbar
+ * @param session The girara session
+ * @return true if no error occurred and the event has been handled
+ */
+gboolean cb_sc_copy_link(GtkEntry* entry, void* session);
+
 /**
  * Emitted when file has been changed
  *
