@@ -260,6 +260,40 @@ handle_goto_page(zathura_t* zathura, GVariant* parameters,
   g_dbus_method_invocation_return_value(invocation, result);
 }
 
+typedef struct {
+  zathura_t*      zathura;
+  girara_list_t** rectangles;
+  unsigned int    page;
+  unsigned int    number_of_pages;
+} highlights_rect_data_t;
+
+static gboolean
+synctex_highlight_rects_impl(gpointer ptr)
+{
+  highlights_rect_data_t* data = ptr;
+
+  synctex_highlight_rects(data->zathura, data->page, data->rectangles);
+
+  for (unsigned int i = 0; i != data->number_of_pages; ++i) {
+    girara_list_free(data->rectangles[i]);
+  }
+  g_free(data->rectangles);
+  return false;
+}
+
+static void
+synctex_highlight_rects_idle(zathura_t* zathura, girara_list_t** rectangles,
+                             unsigned int page, unsigned number_of_pages)
+{
+  highlights_rect_data_t* data = g_try_malloc0(sizeof(highlights_rect_data_t));
+  data->zathura                = zathura;
+  data->rectangles             = rectangles;
+  data->page                   = page;
+  data->number_of_pages        = number_of_pages;
+
+  gdk_threads_add_idle(synctex_highlight_rects_impl, data);
+}
+
 static void
 handle_highlight_rects(zathura_t* zathura, GVariant* parameters,
                        GDBusMethodInvocation* invocation)
@@ -357,26 +391,55 @@ handle_highlight_rects(zathura_t* zathura, GVariant* parameters,
   }
   g_variant_iter_free(secondary_iter);
 
-  synctex_highlight_rects(zathura, page, rectangles);
-  g_free(rectangles);
+  /* run synctex_highlight_rects in main thread when idle */
+  synctex_highlight_rects_idle(zathura, rectangles, page, number_of_pages);
 
   GVariant* result = g_variant_new("(b)", true);
   g_dbus_method_invocation_return_value(invocation, result);
 }
 
+typedef struct {
+  zathura_t*   zathura;
+  gchar*       input_file;
+  unsigned int line;
+  unsigned int column;
+} view_data_t;
+
+static gboolean
+synctex_view_impl(gpointer ptr)
+{
+  view_data_t* data = ptr;
+
+  synctex_view(data->zathura, data->input_file, data->line, data->column);
+
+  g_free(data->input_file);
+  g_free(data);
+  return false;
+}
+
 static void
-handle_synctex_view(zathura_t* zathura, GVariant* parameters,
-                    GDBusMethodInvocation* invocation)
+synctex_view_idle(zathura_t* zathura, gchar* input_file, unsigned int line, unsigned int column)
+{
+  view_data_t* data = g_try_malloc0(sizeof(view_data_t));
+  data->zathura     = zathura;
+  data->input_file  = input_file;
+  data->line        = line;
+  data->column      = column;
+
+  gdk_threads_add_idle(synctex_view_impl, data);
+}
+
+static void
+handle_synctex_view(zathura_t* zathura, GVariant* parameters, GDBusMethodInvocation* invocation)
 {
   gchar* input_file = NULL;
-  guint line = 0;
-  guint column = 0;
+  guint  line       = 0;
+  guint  column     = 0;
   g_variant_get(parameters, "(suu)", &input_file, &line, &column);
 
-  const bool ret = synctex_view(zathura, input_file, line, column);
-  g_free(input_file);
+  synctex_view_idle(zathura, input_file, line, column);
 
-  GVariant* result = g_variant_new("(b)", ret);
+  GVariant* result = g_variant_new("(b)", true);
   g_dbus_method_invocation_return_value(invocation, result);
 }
 
@@ -586,11 +649,10 @@ iterate_instances_call_synctex_view(const char* filename,
 }
 
 int
-zathura_dbus_synctex_position(const char* filename, const char* input_file,
-                              int line, int column, pid_t hint)
+zathura_dbus_synctex_position(const char* filename, const char* input_file, int line, int column, pid_t hint)
 {
   if (filename == NULL || input_file == NULL || line < 0 || column < 0) {
-    return false;
+    return -1;
   }
 
   return iterate_instances_call_synctex_view(filename, input_file, line, column, hint);
